/* *
 * This sample demonstrates handling intents from an Alexa skill using the Alexa Skills Kit SDK (v2).
 * Please visit https://alexa.design/cookbook for additional examples on implementing slots, dialog management,
 * session persistence, api calls, and more.
 * */
const Alexa = require('ask-sdk-core');
const database = require('./database');

function RandomDigimon() {
    const randomIndex = Math.floor(Math.random() * 150);
    return JSON.stringify(database[randomIndex]);
}


function getDigimonByNome(name) {
    let poke = database.find(Digimon => Digimon.name.toUpperCase() === name.toUpperCase())
    console.log(poke)
    if (poke !== null) {
        return JSON.stringify(poke);
    } else {
        return "";
    }
}

function getDigimonByType(tipo) {
  let poke = []
    database.forEach((digimon) => {
        if (Digimon => Digimon.tipo.toUpperCase() === tipo.toUpperCase())
            poke.push(digimon)
    })
    if (poke !== null) {
        const randomIndex = Math.floor(Math.random() * poke.length);
        return JSON.stringify(poke[randomIndex]);
    } else {
        return "";
    }
}


//Azione che avviene all'avvio della skill
const LaunchRequestHandler = {
    canHandle(handlerInput) {
        //Grazie all'sdk di alexa che stiamo utilizzando ci permette di utilizzare una programmazione ad alto livello
        //Come detto Alexa Interagisce tramite JSON , l sdk ci peremtte di avere un modello già gestito semplificando il encodeURIComponent
        // In questo caso il codice sta interrogando il json e capisce che è una tipologia di LaunchRequest
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'LaunchRequest';
    },
    handle(handlerInput) {
        const speakOutput = 'Ciao se vuoi posso dirti un nome di Digimon a caso,vuoi provare?';
        const repromptMsg = 'parla più forte non ho sentito neiente'
        return handlerInput.responseBuilder
            .speak(speakOutput)
            // il metodo reprompt fa capire ad alexa che deve rimanare in attessa che l'utente dica qualcosa
            .reprompt(repromptMsg)
            .getResponse();
    }
};
// Processando la frase si passa da un LaunchRequest ad un IntentRequest
// Se ricordiamo gli intent visti prima alla frase Dammi un Digimon a casa attivava l HelloWorldIntent 
const HelloWorldIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'HelloWorldIntent';
    },
    handle(handlerInput) {
        const speakOutput = 'ciao sono l intent HelloWorldIntent';

        return handlerInput.responseBuilder
            .speak(speakOutput)
            //.reprompt('add a reprompt if you want to keep the session open for the user to respond')
            .getResponse();
    }
};

//IMPORTANTE SCORRI VERSO IL BASSO
const RandomDigimonHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'DigimonACaso';
    },
    handle(handlerInput) {
        const speakOutput = 'il Digimon che ho scelto per te è ' + RandomDigimon();
        const repromptMsg = 'puoi chiedere un altro Digimon oppure dire chiudi per uscire';
        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt(repromptMsg)
            .getResponse();
    }
};


const ScegliDigimonIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'ScegliDigimon';
    },
    handle(handlerInput) {

        var nomeSlot = handlerInput.requestEnvelope.request.intent.slots.nome.value;

        var speakOutput = "";
        if (nomeSlot !== null) {
            speakOutput = getDigimonByNome(nomeSlot) + "  nome Puoi chiedere le caratteristiche di un altro Digimon.";

        } else {
            speakOutput = "Non ho trovato il segno Digimon che hai chiesto. Prova a chiedere di nuovo.";

        }

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt("Puoi chiedere un altro Digimon o dire chiudi per finire.")
            .getResponse();
    }
};

const ScegliDigimonByTipoIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'DammiUnDigimonDalTipo';
    },
    handle(handlerInput) {

        var tipo = handlerInput.requestEnvelope.request.intent.slots.tipo.value;

        var speakOutput = "";
        if (tipo !== null) {
            speakOutput = getDigimonByType(tipo) + "  tipo Puoi chiedere le caratteristiche di un altro Digimon.";

        } else {
            speakOutput = "Non ho trovato il segno Digimon che hai chiesto. Prova a chiedere di nuovo.";

        }

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt("Puoi chiedere un altro Digimon o dire chiudi per finire.")
            .getResponse();
    }
};

const HelpIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'AMAZON.HelpIntent';
    },
    handle(handlerInput) {
        const speakOutput = 'ciao Posso aiutarti?';

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt(speakOutput)
            .getResponse();
    }
};

const CancelAndStopIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            (Alexa.getIntentName(handlerInput.requestEnvelope) === 'AMAZON.CancelIntent' ||
                Alexa.getIntentName(handlerInput.requestEnvelope) === 'AMAZON.StopIntent');
    },
    handle(handlerInput) {
        const speakOutput = 'Ciao! Grazie per aver giocato con noi';

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .getResponse();
    }
};
/* *
 * FallbackIntent triggers when a customer says something that doesn’t map to any intents in your skill
 * It must also be defined in the language model (if the locale supports it)
 * This handler can be safely added but will be ingnored in locales that do not support it yet 
 * */
const FallbackIntentHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest' &&
            Alexa.getIntentName(handlerInput.requestEnvelope) === 'AMAZON.FallbackIntent';
    },
    handle(handlerInput) {
        const speakOutput = 'Sorry, I don\'t know about that. Please try again.';

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt(speakOutput)
            .getResponse();
    }
};
/* *
 * SessionEndedRequest notifies that a session was ended. This handler will be triggered when a currently open 
 * session is closed for one of the following reasons: 1) The user says "exit" or "quit". 2) The user does not 
 * respond or says something that does not match an intent defined in your voice model. 3) An error occurs 
 * */
const SessionEndedRequestHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'SessionEndedRequest';
    },
    handle(handlerInput) {
        console.log(`~~~~ Session ended: ${JSON.stringify(handlerInput.requestEnvelope)}`);
        // Any cleanup logic goes here.
        return handlerInput.responseBuilder.getResponse(); // notice we send an empty response
    }
};
/* *
 * The intent reflector is used for interaction model testing and debugging.
 * It will simply repeat the intent the user said. You can create custom handlers for your intents 
 * by defining them above, then also adding them to the request handler chain below 
 * */
const IntentReflectorHandler = {
    canHandle(handlerInput) {
        return Alexa.getRequestType(handlerInput.requestEnvelope) === 'IntentRequest';
    },
    handle(handlerInput) {
        const intentName = Alexa.getIntentName(handlerInput.requestEnvelope);
        const speakOutput = `You just triggered ${intentName}`;

        return handlerInput.responseBuilder
            .speak(speakOutput)
            //.reprompt('add a reprompt if you want to keep the session open for the user to respond')
            .getResponse();
    }
};
/**
 * Generic error handling to capture any syntax or routing errors. If you receive an error
 * stating the request handler chain is not found, you have not implemented a handler for
 * the intent being invoked or included it in the skill builder below 
 * */
const ErrorHandler = {
    canHandle() {
        return true;
    },
    handle(handlerInput, error) {
        const speakOutput = 'Sorry, I had trouble doing what you asked. Please try again.';
        console.log(`~~~~ Error handled: ${JSON.stringify(error)}`);

        return handlerInput.responseBuilder
            .speak(speakOutput)
            .reprompt(speakOutput)
            .getResponse();
    }
};


/**
 * Questa parte di codice è molto IMPORTANTE in qunato comunica ad alexa quale Handler deve esportare
 * In soldoni se non aggiungiamo il nostro NUOVO Handler qui non funziona
 * */
exports.handler = Alexa.SkillBuilders.custom()
    .addRequestHandlers(
        LaunchRequestHandler,
        RandomDigimonHandler,
        ScegliDigimonIntentHandler,
        ScegliDigimonByTipoIntentHandler,
        HelloWorldIntentHandler,
        HelpIntentHandler,
        CancelAndStopIntentHandler,
        FallbackIntentHandler,
        SessionEndedRequestHandler,
        IntentReflectorHandler)
    .addErrorHandlers(
        ErrorHandler)
    .withCustomUserAgent('sample/hello-world/v1.2')
    .lambda();